#include "io.h"
#include "ssd192x.h"
#include "jpeg_enc.h"
#include "jpeg_dec.h"
#include "camera.h"

#include "include/test_routine.h"
#include "utils.h"
#include "uart0.h"

#include <TinyFS\tff.h>
#include <string.h>

UINT8 filecnt=0;

/************************************************************************************************************
* void SSD1921DVK_JPEG_Encode_Test(void)                                                                                                                                                                                    *
*                                                                                                                                                                                                                      *
* Description:                                                                                                                                                                                                *
*                                                                                                                                                                                                                      *
*                                                                                                                                                                                                                      *
*                                                                                                                                                                                                                      *
*************************************************************************************************************/

void SSD1921DVK_JPEG_Encode_Test(void)
{

	UINT8 key;
	char err = 0;;

	ssd192x_cam_jpeg_pause(0);	// no pause

	myprintf("Choose a key to encode preview to JPEG.\n\r");
	myprintf("Press 'm' to pause preview & encode preview to JPEG.\n\r");
	myprintf("Press 'n' to encode currently preview to JPEG with no pause.\n\r");
	myprintf("Press any other key to exit.\n\n\r");

	key = UART0_GetByte();
	if(key == 'm')
	{
		ssd192x_cam_jpeg_pause(1);
	    myprintf("Start encode process in pause mode...\n\r");
	    err = JPEG_encoding();
	    ssd192x_cam_jpeg_pause(0);
	}
	else if(key == 'n')
	{
		ssd192x_cam_jpeg_pause(0);
	    myprintf("Start encode process in no pause mode....\n\r");
	    err = JPEG_encoding();
	}

	if (err !=0)
		myprintf("Error in JPEG encoding !\n\r");
}

char JPEG_encoding(void)
{
	UINT32 filesize = 0;
	UINT32 EXPECTED_JPEG_RESOLUTION_WIDTH ;
	UINT32 EXPECTED_JPEG_RESOLUTION_HEIGHT;
	UINT8 key, error, retry;

	error = 0;

	ssd192x_jpeg_init();

	myprintf("\nSelect JPEG output resolution:\r\n");
	myprintf("1. 640x480\r\n");
	myprintf("2. 320x240\r\n");
	myprintf("3. 160x120\r\n");
	myprintf("4. 176x220\r\n");
	myprintf("5. 600x450\r\n");
	myprintf("6. 400x300\r\n");
	key = UART0_GetByte();
	myprintf("Your choice is %c\r\n\n", key);
	switch(key)
	{
		case '1':
			EXPECTED_JPEG_RESOLUTION_WIDTH = 640;
			EXPECTED_JPEG_RESOLUTION_HEIGHT = 480;
			break;
		case '2':
			EXPECTED_JPEG_RESOLUTION_WIDTH = 320;
			EXPECTED_JPEG_RESOLUTION_HEIGHT = 240;
			break;
		case '3':
			EXPECTED_JPEG_RESOLUTION_WIDTH = 160;
			EXPECTED_JPEG_RESOLUTION_HEIGHT = 120;
			break;
		case '4':
			EXPECTED_JPEG_RESOLUTION_WIDTH = 176;
			EXPECTED_JPEG_RESOLUTION_HEIGHT = 220;
			break;
		case '5':
			EXPECTED_JPEG_RESOLUTION_WIDTH = 600;
			EXPECTED_JPEG_RESOLUTION_HEIGHT = 450;
			break;
		case '6':
			EXPECTED_JPEG_RESOLUTION_WIDTH = 400;
			EXPECTED_JPEG_RESOLUTION_HEIGHT = 300;
			break;
		default:
			// the input size of the CMOS sensor
			EXPECTED_JPEG_RESOLUTION_WIDTH = SSD_REGRW(REG_DV_HCROP_SIZE_0);
			EXPECTED_JPEG_RESOLUTION_HEIGHT = SSD_REGRW(REG_DV_VCROP_SIZE_0);
			break;
	}

	myprintf("The encoded resolution is : %d x %d\r\n",EXPECTED_JPEG_RESOLUTION_WIDTH,EXPECTED_JPEG_RESOLUTION_HEIGHT);
	Set_JPEG_Encode_Registers(EXPECTED_JPEG_RESOLUTION_WIDTH,EXPECTED_JPEG_RESOLUTION_HEIGHT);

	filesize = Start_JPEG_Encode();
	if ( filesize == 0)
		return -1;

	retry = 0;
	if(filecnt == 0)
		filecnt = 0;	// keep track of number of files encoded

	myprintf("1. Write to SD card \r\n");
	myprintf("2. Write to 192x internal memory\r\n");

	myprintf("Press any other key to exit without saving\r\n");
	key = UART0_GetByte();

	myprintf("Your choice is %c\r\n\n", key);

	switch(key)
	{
		case '1' :

			myprintf("Writing to SD card attempt = %d, 0x%x\r\n", retry, filesize);
			// Attempt to write to the SD card 3 times
			while ( retry < 3)
			{
				error = Write_JPEG_data_to_SD(filesize, filecnt);	// code at jpegenc.c
				if ( error != 0 )
				{
					retry++;
				}
				else
				{
					myprintf("Write to SD successful\r\n");
					Off_jpeg();
					filecnt++;
					return 0;
				}

			}
			myprintf("Error writing file to SD Card !\r\n");
			break;

		case '2' :
			Write_JPEG_data_to_192x(filesize);	// code at jpegenc.c
	}

	Off_jpeg();
	return error;
}
